@@ -9,7 +9,9 @@
  *   # define N_(String) (String)
  *   #endif
  *
- * Copyright 2010 Andrew Wood, distributed under the Artistic License 2.0.
+ * Copyright 2002-2008, 2010, 2012-2015, 2017, 2021, 2023 Andrew Wood
+ *
+ * Distributed under the Artistic License v2.0; see `doc/COPYING'.
  */
 
 #ifndef _LIBRARY_GETTEXT_H
@@ -27,22 +29,26 @@
 #else
 # define _(String)	minigettext (String)
 # define N_(String)	(String)
-# define setlocale	minisetlocale
+# ifdef HAVE_LOCALE_H
+#  include <locale.h>
+# else
+#  define setlocale	minisetlocale
+#  ifndef LC_ALL
+#   define LC_ALL	""
+#  endif
+# endif
 # define bindtextdomain	minibindtextdomain
 # define textdomain	minitextdomain
-# ifndef LC_ALL
-#  define LC_ALL	""
-# endif
 #endif
 
 #ifdef __cplusplus
 extern "C" {
 #endif
 
-char *minisetlocale(char *, char *);
-char *minibindtextdomain(char *, char *);
-char *minitextdomain(char *);
-char *minigettext(char *);
+char *minisetlocale(const char *, const char *);
+char *minibindtextdomain(const char *, const char *);
+char *minitextdomain(const char *);
+char *minigettext(const char *);
 
 #ifdef __cplusplus
 }
