@@ -12,7 +12,9 @@
  * of the table being marked by a NULL msgid. The po2table.sh script will do
  * this.
  *
- * Copyright 2010 Andrew Wood, distributed under the Artistic License 2.0.
+ * Copyright 2002-2008, 2010, 2012-2015, 2017, 2021, 2023 Andrew Wood
+ *
+ * Distributed under the Artistic License v2.0; see `doc/COPYING'.
  */
 
 #include "config.h"
@@ -36,53 +38,53 @@ struct msgtable_s *minigettext__gettable(char *a)
 }
 #endif				/* ENABLE_NLS */
 
-char *minisetlocale(char *a, char *b)
+char *minisetlocale(const char *a, const char *b)
 {
 	return NULL;
 }
 
 
-char *minibindtextdomain(char *a, char *b)
+char *minibindtextdomain(const char *a, const char *b)
 {
 	return NULL;
 }
 
 
-char *minitextdomain(char *a)
+char *minitextdomain(const char *a)
 {
 	return NULL;
 }
 
 
-char *minigettext(char *msgid)
+char *minigettext(const char *msgid)
 {
 	static struct msgtable_s *table = NULL;
 	static int tried_lang = 0;
 	char *lang;
 	int i;
 
-	if (msgid == NULL)
-		return msgid;
+	if (NULL == msgid)
+		return NULL;
 
-	if (tried_lang == 0) {
-		lang = getenv("LANGUAGE");  /* RATS: ignore */
+	if (0 == tried_lang) {
+		lang = getenv("LANGUAGE");
 		if (lang)
 			table = minigettext__gettable(lang);
 
-		if (table == NULL) {
-			lang = getenv("LANG");	/* RATS: ignore */
+		if (NULL == table) {
+			lang = getenv("LANG");
 			if (lang)
 				table = minigettext__gettable(lang);
 		}
 
-		if (table == NULL) {
-			lang = getenv("LC_ALL");	/* RATS: ignore */
+		if (NULL == table) {
+			lang = getenv("LC_ALL");
 			if (lang)
 				table = minigettext__gettable(lang);
 		}
 
-		if (table == NULL) {
-			lang = getenv("LC_MESSAGES");	/* RATS: ignore */
+		if (NULL == table) {
+			lang = getenv("LC_MESSAGES");
 			if (lang)
 				table = minigettext__gettable(lang);
 		}
@@ -90,20 +92,20 @@ char *minigettext(char *msgid)
 		tried_lang = 1;
 	}
 
-	if (table == NULL)
-		return msgid;
+	if (NULL == table)
+		return (char *) msgid;
 
 	for (i = 0; table[i].msgid; i++) {
-		if (strcmp(table[i].msgid, msgid) == 0) {
-			if (table[i].msgstr == 0)
-				return msgid;
-			if (table[i].msgstr[0] == 0)
-				return msgid;
+		if (0 == strcmp(table[i].msgid, msgid)) {
+			if (0 == table[i].msgstr)
+				return (char *) msgid;
+			if (0 == table[i].msgstr[0])
+				return (char *) msgid;
 			return table[i].msgstr;
 		}
 	}
 
-	return msgid;
+	return (char *) msgid;
 }
 
 #endif				/* HAVE_GETTEXT */
