@@ -1,12 +1,15 @@
 /*
  * Functions for converting strings to numbers.
  *
- * Copyright 2010 Andrew Wood, distributed under the Artistic License 2.0.
+ * Copyright 2002-2008, 2010, 2012-2015, 2017, 2021, 2023 Andrew Wood
+ *
+ * Distributed under the Artistic License v2.0; see `doc/COPYING'.
  */
 
-#ifdef HAVE_CONFIG_H
 #include "config.h"
-#endif
+#include "pv.h"
+
+#include <stddef.h>
 
 
 /*
@@ -14,39 +17,42 @@
  * including <ctype.h> causes weird versioned glibc dependencies on certain
  * Red Hat systems, complicating package management.
  */
-static int pv__isdigit(char c)
+static bool pv__isdigit(char c)
 {
-	return ((c >= '0') && (c <= '9'));
+	return ((c >= '0') && (c <= '9')) ? true : false;
 }
 
 
 /*
- * Return the numeric value of "str", as a long long.
+ * Return the numeric value of "str", as an unsigned long long.
  */
-long long pv_getnum_ll(char *str)
+unsigned long long pv_getnum_ull(const char *str)
 {
-	long long n = 0;
-	long long decimal = 0;
-	int decdivisor = 1;
-	int shift = 0;
+	unsigned long long n = 0;
+	unsigned long long decimal = 0;
+	unsigned int decdivisor = 1;
+	unsigned int shift = 0;
 
-	while (str[0] != 0 && (!pv__isdigit(str[0])))
+	if (NULL == str)
+		return n;
+
+	while (str[0] != '\0' && (!pv__isdigit(str[0])))
 		str++;
 
 	for (; pv__isdigit(str[0]); str++) {
 		n = n * 10;
-		n += (str[0] - '0');
+		n += (unsigned long long) (str[0] - '0');
 	}
 
 	/*
 	 * If a decimal value was given, skip the decimal part.
 	 */
-	if ((str[0] == '.') || (str[0] == ',')) {
+	if (('.' == str[0]) || (',' == str[0])) {
 		str++;
 		for (; pv__isdigit(str[0]); str++) {
 			if (decdivisor < 10000) {
 				decimal = decimal * 10;
-				decimal += (str[0] - '0');
+				decimal += (unsigned long long) (str[0] - '0');
 				decdivisor = decdivisor * 10;
 			}
 		}
@@ -56,8 +62,8 @@ long long pv_getnum_ll(char *str)
 	 * Parse any units given (K=KiB=*1024, M=MiB=1024KiB, G=GiB=1024MiB,
 	 * T=TiB=1024GiB).
 	 */
-	if (str[0]) {
-		while ((str[0] == ' ') || (str[0] == '\t'))
+	if (str[0] != '\0') {
+		while ((' ' == str[0]) || ('\t' == str[0]))
 			str++;
 		switch (str[0]) {
 		case 'k':
@@ -87,7 +93,7 @@ long long pv_getnum_ll(char *str)
 	 * more than 30 at a time to avoid overflows.
 	 */
 	while (shift > 0) {
-		int shiftby;
+		unsigned int shiftby;
 
 		shiftby = shift;
 		if (shiftby > 30)
@@ -111,17 +117,20 @@ long long pv_getnum_ll(char *str)
 /*
  * Return the numeric value of "str", as a double.
  */
-double pv_getnum_d(char *str)
+double pv_getnum_d(const char *str)
 {
 	double n = 0.0;
 	double step = 1;
 
-	while (str[0] != 0 && (!pv__isdigit(str[0])))
+	if (NULL == str)
+		return n;
+
+	while (str[0] != '\0' && (!pv__isdigit(str[0])))
 		str++;
 
 	for (; pv__isdigit(str[0]); str++) {
 		n = n * 10;
-		n += (str[0] - '0');
+		n += (double) (str[0] - '0');
 	}
 
 	if ((str[0] != '.') && (str[0] != ','))
@@ -131,7 +140,7 @@ double pv_getnum_d(char *str)
 
 	for (; pv__isdigit(str[0]) && step < 1000000; str++) {
 		step = step * 10;
-		n += (str[0] - '0') / step;
+		n += ((double) (str[0] - '0')) / step;
 	}
 
 	return n;
@@ -139,24 +148,24 @@ double pv_getnum_d(char *str)
 
 
 /*
- * Return the numeric value of "str", as an int.
+ * Return the numeric value of "str", as an unsigned int.
  */
-int pv_getnum_i(char *str)
+unsigned int pv_getnum_ui(const char *str)
 {
-	return (int) pv_getnum_ll(str);
+	return (unsigned int) pv_getnum_ull(str);
 }
 
 
 /*
- * Return nonzero if the given string is not a valid integer (type=0) or
- * double (type=1).
+ * Return nonzero if the given string is not a valid number of the given
+ * type.
  */
-int pv_getnum_check(char *str, int type)
+int pv_getnum_check(const char *str, pv_numtype_t type)
 {
-	if (!str)
+	if (0 == str)
 		return 1;
 
-	while ((str[0] == ' ') || (str[0] == '\t'))
+	while ((' ' == str[0]) || ('\t' == str[0]))
 		str++;
 
 	if (!pv__isdigit(str[0]))
@@ -164,23 +173,23 @@ int pv_getnum_check(char *str, int type)
 
 	for (; pv__isdigit(str[0]); str++);
 
-	if (str[0] == '.') {
-		if (type == 0)
+	if (('.' == str[0]) || (',' == str[0])) {
+		if (type == PV_NUMTYPE_INTEGER)
 			return 1;
 		str++;
 		for (; pv__isdigit(str[0]); str++);
 	}
 
-	if (str[0] == 0)
+	if ('\0' == str[0])
 		return 0;
 
 	/*
 	 * Suffixes are not allowed for doubles, only for integers.
 	 */
-	if (type == 1)
+	if (type == PV_NUMTYPE_DOUBLE)
 		return 1;
 
-	while ((str[0] == ' ') || (str[0] == '\t'))
+	while ((' ' == str[0]) || ('\t' == str[0]))
 		str++;
 	switch (str[0]) {
 	case 'k':
@@ -197,7 +206,7 @@ int pv_getnum_check(char *str, int type)
 		return 1;
 	}
 
-	if (str[0])
+	if (str[0] != '\0')
 		return 1;
 
 	return 0;
