@@ -1,35 +1,28 @@
 /*
  * Signal handling functions.
  *
- * Copyright 2010 Andrew Wood, distributed under the Artistic License 2.0.
+ * Copyright 2002-2008, 2010, 2012-2015, 2017, 2021, 2023 Andrew Wood
+ *
+ * Distributed under the Artistic License v2.0; see `doc/COPYING'.
  */
 
+#include "config.h"
 #include "pv.h"
+#include "pv-internal.h"
 
 #include <signal.h>
 #include <termios.h>
 #include <time.h>
 #include <unistd.h>
-#include <sys/types.h>
 #include <sys/stat.h>
 #include <fcntl.h>
-#include <sys/time.h>
-
-#ifdef HAVE_CONFIG_H
-#include "config.h"
-#endif
-
-static int pv__sig_old_stderr;		 /* see pv__sig_ttou() */
-static struct timeval pv__sig_tstp_time; /* see pv__sig_tstp() / __cont() */
-
-struct timeval pv_sig_toffset;		 /* total time spent stopped */
-sig_atomic_t pv_sig_newsize = 0;	 /* whether we need to get term size again */
-sig_atomic_t pv_sig_abort = 0;		 /* whether we need to abort right now */
 
 #ifdef HAVE_IPC
-void pv_crs_needreinit(void);
+void pv_crs_needreinit(pvstate_t);
 #endif
 
+static pvstate_t pv_sig_state = NULL;
+
 
 /*
  * Handle SIGTTOU (tty output for background process) by redirecting stderr
@@ -38,16 +31,17 @@ void pv_crs_needreinit(void);
  * subsequent SIGCONT we can try writing to the terminal again, in case we
  * get backgrounded and later get foregrounded again.
  */
-static void pv__sig_ttou(int s)
+static void pv_sig_ttou( __attribute__((unused))
+			int s)
 {
 	int fd;
 
-	fd = open("/dev/null", O_RDWR);	    /* RATS: ignore (no race) */
+	fd = open("/dev/null", O_RDWR);
 	if (fd < 0)
 		return;
 
-	if (pv__sig_old_stderr == -1)
-		pv__sig_old_stderr = dup(STDERR_FILENO);
+	if (-1 == pv_sig_state->pv_sig_old_stderr)
+		pv_sig_state->pv_sig_old_stderr = dup(STDERR_FILENO);
 
 	dup2(fd, STDERR_FILENO);
 	close(fd);
@@ -56,11 +50,12 @@ static void pv__sig_ttou(int s)
 
 /*
  * Handle SIGTSTP (stop typed at tty) by storing the time the signal
- * happened for later use by pv__sig_cont(), and then stopping the process.
+ * happened for later use by pv_sig_cont(), and then stopping the process.
  */
-static void pv__sig_tstp(int s)
+static void pv_sig_tstp( __attribute__((unused))
+			int s)
 {
-	gettimeofday(&pv__sig_tstp_time, NULL);
+	gettimeofday(&(pv_sig_state->pv_sig_tstp_time), NULL);
 	raise(SIGSTOP);
 }
 
@@ -70,43 +65,44 @@ static void pv__sig_tstp(int s)
  * last SIGTSTP to the elapsed time offset, and by trying to write to the
  * terminal again (by replacing the /dev/null stderr with the old stderr).
  */
-static void pv__sig_cont(int s)
+static void pv_sig_cont( __attribute__((unused))
+			int s)
 {
 	struct timeval tv;
 	struct termios t;
 
-	pv_sig_newsize = 1;
+	pv_sig_state->pv_sig_newsize = 1;
 
-	if (pv__sig_tstp_time.tv_sec == 0) {
+	if (0 == pv_sig_state->pv_sig_tstp_time.tv_sec) {
 		tcgetattr(STDERR_FILENO, &t);
 		t.c_lflag |= TOSTOP;
 		tcsetattr(STDERR_FILENO, TCSANOW, &t);
 #ifdef HAVE_IPC
-		pv_crs_needreinit();
+		pv_crs_needreinit(pv_sig_state);
 #endif
 		return;
 	}
 
 	gettimeofday(&tv, NULL);
 
-	pv_sig_toffset.tv_sec += (tv.tv_sec - pv__sig_tstp_time.tv_sec);
-	pv_sig_toffset.tv_usec += (tv.tv_usec - pv__sig_tstp_time.tv_usec);
-	if (pv_sig_toffset.tv_usec >= 1000000) {
-		pv_sig_toffset.tv_sec++;
-		pv_sig_toffset.tv_usec -= 1000000;
+	pv_sig_state->pv_sig_toffset.tv_sec += (tv.tv_sec - pv_sig_state->pv_sig_tstp_time.tv_sec);
+	pv_sig_state->pv_sig_toffset.tv_usec += (tv.tv_usec - pv_sig_state->pv_sig_tstp_time.tv_usec);
+	if (pv_sig_state->pv_sig_toffset.tv_usec >= 1000000) {
+		pv_sig_state->pv_sig_toffset.tv_sec++;
+		pv_sig_state->pv_sig_toffset.tv_usec -= 1000000;
 	}
-	if (pv_sig_toffset.tv_usec < 0) {
-		pv_sig_toffset.tv_sec--;
-		pv_sig_toffset.tv_usec += 1000000;
+	if (pv_sig_state->pv_sig_toffset.tv_usec < 0) {
+		pv_sig_state->pv_sig_toffset.tv_sec--;
+		pv_sig_state->pv_sig_toffset.tv_usec += 1000000;
 	}
 
-	pv__sig_tstp_time.tv_sec = 0;
-	pv__sig_tstp_time.tv_usec = 0;
+	pv_sig_state->pv_sig_tstp_time.tv_sec = 0;
+	pv_sig_state->pv_sig_tstp_time.tv_usec = 0;
 
-	if (pv__sig_old_stderr != -1) {
-		dup2(pv__sig_old_stderr, STDERR_FILENO);
-		close(pv__sig_old_stderr);
-		pv__sig_old_stderr = -1;
+	if (pv_sig_state->pv_sig_old_stderr != -1) {
+		dup2(pv_sig_state->pv_sig_old_stderr, STDERR_FILENO);
+		close(pv_sig_state->pv_sig_old_stderr);
+		pv_sig_state->pv_sig_old_stderr = -1;
 	}
 
 	tcgetattr(STDERR_FILENO, &t);
@@ -114,7 +110,7 @@ static void pv__sig_cont(int s)
 	tcsetattr(STDERR_FILENO, TCSANOW, &t);
 
 #ifdef HAVE_IPC
-	pv_crs_needreinit();
+	pv_crs_needreinit(pv_sig_state);
 #endif
 }
 
@@ -122,33 +118,37 @@ static void pv__sig_cont(int s)
 /*
  * Handle SIGWINCH (window size changed) by setting a flag.
  */
-static void pv__sig_winch(int s)
+static void pv_sig_winch( __attribute__((unused))
+			 int s)
 {
-	pv_sig_newsize = 1;
+	pv_sig_state->pv_sig_newsize = 1;
 }
 
 
 /*
  * Handle termination signals by setting the abort flag.
  */
-static void pv__sig_term(int s)
+static void pv_sig_term( __attribute__((unused))
+			int s)
 {
-	pv_sig_abort = 1;
+	pv_sig_state->pv_sig_abort = 1;
 }
 
 
 /*
  * Initialise signal handling.
  */
-void pv_sig_init(void)
+void pv_sig_init(pvstate_t state)
 {
 	struct sigaction sa;
 
-	pv__sig_old_stderr = -1;
-	pv__sig_tstp_time.tv_sec = 0;
-	pv__sig_tstp_time.tv_usec = 0;
-	pv_sig_toffset.tv_sec = 0;
-	pv_sig_toffset.tv_usec = 0;
+	pv_sig_state = state;
+
+	pv_sig_state->pv_sig_old_stderr = -1;
+	pv_sig_state->pv_sig_tstp_time.tv_sec = 0;
+	pv_sig_state->pv_sig_tstp_time.tv_usec = 0;
+	pv_sig_state->pv_sig_toffset.tv_sec = 0;
+	pv_sig_state->pv_sig_toffset.tv_usec = 0;
 
 	/*
 	 * Ignore SIGPIPE, so we don't die if stdout is a pipe and the other
@@ -157,63 +157,79 @@ void pv_sig_init(void)
 	sa.sa_handler = SIG_IGN;
 	sigemptyset(&(sa.sa_mask));
 	sa.sa_flags = 0;
-	sigaction(SIGPIPE, &sa, NULL);
+	sigaction(SIGPIPE, &sa, &(pv_sig_state->pv_sig_old_sigpipe));
 
 	/*
 	 * Handle SIGTTOU by continuing with output switched off, so that we
 	 * can be stopped and backgrounded without messing up the terminal.
 	 */
-	sa.sa_handler = pv__sig_ttou;
+	sa.sa_handler = pv_sig_ttou;
 	sigemptyset(&(sa.sa_mask));
 	sa.sa_flags = 0;
-	sigaction(SIGTTOU, &sa, NULL);
+	sigaction(SIGTTOU, &sa, &(pv_sig_state->pv_sig_old_sigttou));
 
 	/*
 	 * Handle SIGTSTP by storing the time the signal happened for later
-	 * use by pv__sig_cont(), and then stopping the process.
+	 * use by pv_sig_cont(), and then stopping the process.
 	 */
-	sa.sa_handler = pv__sig_tstp;
+	sa.sa_handler = pv_sig_tstp;
 	sigemptyset(&(sa.sa_mask));
 	sa.sa_flags = 0;
-	sigaction(SIGTSTP, &sa, NULL);
+	sigaction(SIGTSTP, &sa, &(pv_sig_state->pv_sig_old_sigtstp));
 
 	/*
 	 * Handle SIGCONT by adding the elapsed time since the last SIGTSTP
 	 * to the elapsed time offset, and by trying to write to the
 	 * terminal again.
 	 */
-	sa.sa_handler = pv__sig_cont;
+	sa.sa_handler = pv_sig_cont;
 	sigemptyset(&(sa.sa_mask));
 	sa.sa_flags = 0;
-	sigaction(SIGCONT, &sa, NULL);
+	sigaction(SIGCONT, &sa, &(pv_sig_state->pv_sig_old_sigcont));
 
 	/*
 	 * Handle SIGWINCH by setting a flag to let the main loop know it
 	 * has to reread the terminal size.
 	 */
-	sa.sa_handler = pv__sig_winch;
+	sa.sa_handler = pv_sig_winch;
 	sigemptyset(&(sa.sa_mask));
 	sa.sa_flags = 0;
-	sigaction(SIGWINCH, &sa, NULL);
+	sigaction(SIGWINCH, &sa, &(pv_sig_state->pv_sig_old_sigwinch));
 
 	/*
 	 * Handle SIGINT, SIGHUP, SIGTERM by setting a flag to let the
 	 * main loop know it should quit now.
 	 */
-	sa.sa_handler = pv__sig_term;
+	sa.sa_handler = pv_sig_term;
 	sigemptyset(&(sa.sa_mask));
 	sa.sa_flags = 0;
-	sigaction(SIGINT, &sa, NULL);
+	sigaction(SIGINT, &sa, &(pv_sig_state->pv_sig_old_sigint));
 
-	sa.sa_handler = pv__sig_term;
+	sa.sa_handler = pv_sig_term;
 	sigemptyset(&(sa.sa_mask));
 	sa.sa_flags = 0;
-	sigaction(SIGHUP, &sa, NULL);
+	sigaction(SIGHUP, &sa, &(pv_sig_state->pv_sig_old_sighup));
 
-	sa.sa_handler = pv__sig_term;
+	sa.sa_handler = pv_sig_term;
 	sigemptyset(&(sa.sa_mask));
 	sa.sa_flags = 0;
-	sigaction(SIGTERM, &sa, NULL);
+	sigaction(SIGTERM, &sa, &(pv_sig_state->pv_sig_old_sigterm));
+}
+
+
+/*
+ * Shut down signal handling.
+ */
+void pv_sig_fini( __attribute__((unused)) pvstate_t state)
+{
+	sigaction(SIGPIPE, &(pv_sig_state->pv_sig_old_sigpipe), NULL);
+	sigaction(SIGTTOU, &(pv_sig_state->pv_sig_old_sigttou), NULL);
+	sigaction(SIGTSTP, &(pv_sig_state->pv_sig_old_sigtstp), NULL);
+	sigaction(SIGCONT, &(pv_sig_state->pv_sig_old_sigcont), NULL);
+	sigaction(SIGWINCH, &(pv_sig_state->pv_sig_old_sigwinch), NULL);
+	sigaction(SIGINT, &(pv_sig_state->pv_sig_old_sigint), NULL);
+	sigaction(SIGHUP, &(pv_sig_state->pv_sig_old_sighup), NULL);
+	sigaction(SIGTERM, &(pv_sig_state->pv_sig_old_sigterm), NULL);
 }
 
 
@@ -243,12 +259,12 @@ void pv_sig_allowpause(void)
 {
 	struct sigaction sa;
 
-	sa.sa_handler = pv__sig_tstp;
+	sa.sa_handler = pv_sig_tstp;
 	sigemptyset(&(sa.sa_mask));
 	sa.sa_flags = 0;
 	sigaction(SIGTSTP, &sa, NULL);
 
-	sa.sa_handler = pv__sig_cont;
+	sa.sa_handler = pv_sig_cont;
 	sigemptyset(&(sa.sa_mask));
 	sa.sa_flags = 0;
 	sigaction(SIGCONT, &sa, NULL);
@@ -271,19 +287,19 @@ void pv_sig_checkbg(void)
 
 	next_check = time(NULL) + 1;
 
-	if (pv__sig_old_stderr == -1)
+	if (-1 == pv_sig_state->pv_sig_old_stderr)
 		return;
 
-	dup2(pv__sig_old_stderr, STDERR_FILENO);
-	close(pv__sig_old_stderr);
-	pv__sig_old_stderr = -1;
+	dup2(pv_sig_state->pv_sig_old_stderr, STDERR_FILENO);
+	close(pv_sig_state->pv_sig_old_stderr);
+	pv_sig_state->pv_sig_old_stderr = -1;
 
 	tcgetattr(STDERR_FILENO, &t);
 	t.c_lflag |= TOSTOP;
 	tcsetattr(STDERR_FILENO, TCSANOW, &t);
 
 #ifdef HAVE_IPC
-	pv_crs_needreinit();
+	pv_crs_needreinit(pv_sig_state);
 #endif
 }
 
